
/** Show a list of minimized windows on the desktop
**/

// TODO
// * Perhaps limit the length of window title strings
// * Possibly distinguish between windows on different desktops


const St = imports.gi.St;
const Main = imports.ui.main;
const Shell = imports.gi.Shell;
const Meta = imports.gi.Meta;
const Lang = imports.lang;

var _text = null;
var _desktop = null;

/**
* Build a string with the application and title of all minimized windows and
* position it at the bottom left corner of the primary screen.
*/
function _show_text(source)
{
    //======================================//
    // Remove the previous text if present. //
    //======================================//
    if (_text) {
    	_text.destroy();
    	_text = null;
    }

    //========================================================//
    // Build a string containing a list of minimized windows. //
    //========================================================//
    let string = null;
    let _tracker = Shell.WindowTracker.get_default();
    let windows = global.get_window_actors();

    for (let i = 0; i < windows.length; i++) {
    	if (!(windows[i].showing_on_its_workspace()))
	{
		let name = _tracker.get_window_app(windows[i].get_meta_window()).get_name();
		let title = windows[i].get_meta_window().get_title();
		if (string == null)
		{
			string = name+" - "+title;
		}
		else
		{
			string = string + "\n"+name+" - "+title;
		}

	}

    }

    //==========================================================//
    // If the string is not null, add the label to the desktop. //
    //==========================================================//
    if (string != null)
    {
    	_text = new St.Label({ style_class: 'minimisedWindows-label', text: string });

	if (_desktop == null)
	{
		//===================================================//
		// Search for the DESKTOP window in the window list. //
		//===================================================//
		let i;
		for (i=0; i<windows.length; i++)
		{
			if (windows[i].get_meta_window().get_window_type() == Meta.WindowType.DESKTOP)
			{
				_desktop = windows[i];
				break;
			}
		}
		if (i >= windows.length)
		{
			global.log("MinimizedWindowReminder failed to find DESKTOP window");
			//===================================================//
			//If we didn't find it then it's likely the desktop  //
			//isn't managed by the File Manager so assume the    //
			//parent of the first window in the stack            //
			//===================================================//
			if (windows.length > 0) {
				_desktop = windows[0].get_parent();
			}
		}
	}

	//==================================================================//
	// Add the window to the DESKTOP and set it's position and opacity. //
	//==================================================================//
    	_desktop.add_actor(_text);
    	_text.opacity = 128;
	let monitor = Main.layoutManager.primaryMonitor;
    	_text.set_position( 	monitor.width - _text.width - 16,
                		monitor.height - _text.height - 16);

	//Ensure it's positioned above the background
	//(required for when desktop isn't managed by file manager)
	_text.raise(_desktop.get_children()[0]);

    }
}

function init() {
}

function enable() {

	global.screen.connect('restacked', Lang.bind(this, _restacked));

	_show_text("enabled");
}

function _restacked() {

	_show_text("restacked");
}


function disable() {

	//=================================//
	// Unhook from the Window Manager. //
	//=================================//
	global.screen.disconnect(_restacked);

	//=============================//
	// Remove the text if present. //
	//=============================//
	if (_text) {
		_text.destroy();
		_text = null;
	}

}
